import java.io.FileNotFoundException;
import java.util.LinkedList;

import graphics.BasicOceanCanvas;
import ocean.BasicDirections;
import ocean.Coordinate;
import ocean.Direction;
import ocean.Ocean;

/**********************************
 * Question 3 : éviter les requins
 **********************************/

/**
 * Parcours utilisant une Dequeue pour éviter les requins
 */
class AvoidSharks extends BackTrackingTraversal {
	/**
	 * Implémente le parcours évitant les requins
	 */
	@Override
	public boolean traverse(Ocean ocean, Coordinate start) {
		// TODO: implémenter le parcours utilisant une Dequeue pour éviter les requins
		// (Question 3)

		// @OFF
		LinkedList<Coordinate> toBeVisited = new LinkedList<Coordinate>();
		toBeVisited.add(start);
		ocean.setMark(start);

		Coordinate current = null;
		while (!toBeVisited.isEmpty()) {
			current = toBeVisited.poll();
			if (ocean.isNemoAt(current)) {
				backTrack(ocean, current);
				return true;
			}

			for (Direction direction : ocean.directions()) {
				Coordinate neighbour = current.moveTo(direction);

				if (ocean.isValid(neighbour) && !ocean.isWall(neighbour) && !ocean.isMarked(neighbour)) {
					ocean.setMark(neighbour, direction.getOpposite());

					if (ocean.isThereASharkAt(neighbour)) {
						toBeVisited.addLast(neighbour);
					} else {
						toBeVisited.addFirst(neighbour);
					}
				}
			}
		}
		return false;
		/* @ON
		throw new Error("Méthode AvoidSharks.traverse(Ocean, Coordinate) à implémenter (Question 3)");
		@OFF @ON */
	}

	/**
	 * Lance la visualisation du parcours de l'océan dont la carte se trouve dans
	 * {@code one-shark.txt}
	 * 
	 * @param args inutilisé
	 * @throws FileNotFoundException lorsque la carte n'est pas accessible
	 */
	public static void main(String[] args) throws FileNotFoundException {
		String fileName = (args.length < 1) ? "one-shark.txt" : args[0];

		// on crée un nouvel océan à partir de la carte avec requins one-shark.txt
		// et on y associe un nouveau rapporteur qui sera utilisé pour suivre toutes les
		// explorations à la suite
		Ocean sharks = new Ocean(fileName, "data/" + fileName, BasicDirections.values());

		System.out.println("Test evitement des requins (avoid sharks)");
		sharks.reporters().add(new BasicOceanCanvas("Finding Nemo - With Sharks")).exploreUsing(new AvoidSharks());
	}
}
