import java.io.FileNotFoundException;
import java.util.LinkedList;

import graphics.BasicOceanCanvas;
import ocean.BasicDirections;
import ocean.Coordinate;
import ocean.Direction;
import ocean.Ocean;

/**
 * Parcours altérnatif
 */
class AvoidSharksDequeue extends BackTrackingTraversal {
	// Alternative Idea: use 3 queues: one for the current connected component of
	// maps \ sharks, one for upcoming sharks, and one for further sharks not yet
	// considered (still too far away)
	@Override
	public boolean traverse(Ocean ocean, Coordinate start) {
		// TODO: implémenter un parcours qui calcule un chemin de longueur optimale
		// 		 parmi ceux qui rencontrent le moins de requins
		// (Question 4)

		// @OFF
		LinkedList<Pair> now = new LinkedList<>();
		LinkedList<Pair> next = new LinkedList<>();
		LinkedList<Pair> sharks = new LinkedList<>();

		now.add(new Pair(start, 0));
		Pair current = null;

		ocean.setMark(start);
		
		do {
			while (!now.isEmpty() || !sharks.isEmpty()) {
				// look for next cell (transfer shark if necessary)
				if (now.isEmpty())
					now.add(sharks.poll());

				current = now.poll();

				// while distance to next shark is reached, enqueue it
				while (!sharks.isEmpty() && current.counter >= sharks.peek().counter)
					now.add(sharks.poll());

				// test success
				if (ocean.isNemoAt(current.cell)) {
					backTrack(ocean, current.cell);
					return true;
				}

				// enqueue relevant neighboring cells
				for (Direction direction : ocean.directions()) {
					Coordinate neighbour = current.cell.moveTo(direction);

					if (ocean.isValid(neighbour) && !ocean.isWall(neighbour) && !ocean.isMarked(neighbour)) {
						ocean.setMark(neighbour, direction.getOpposite());
						if (ocean.isThereASharkAt(neighbour))
							next.add(new Pair(neighbour, current.counter + 1));
						else
							now.add(new Pair(neighbour, current.counter + 1));
					}
				}
			}
			// upon termination, update sharks and cells queues
			sharks = next;
			next = new LinkedList<Pair>();
			now = new LinkedList<Pair>();
		} while (!sharks.isEmpty());
		return false;
		/* @ON
		throw new Error("Méthode AvoidSharksDequeue.traverse(Ocean, Coordinate) à implémenter (Question 4)");
		@OFF @ON */
	}
	
	/**
	 * Lance la visualisation du parcours de l'océan dont la carte se trouve dans
	 * {@code one-shark.txt}
	 * 
	 * @param args non utilisé
	 * @throws FileNotFoundException lorsque la carte n'est pas accessible
	 */
	public static void main(String[] args) throws FileNotFoundException {
		String fileName = (args.length < 1) ? "one-shark.txt" : args[0];

		// on crée un nouveau océan à partir de la carte avec requins one-shark.txt
		// et on y associe un nouveau rapporteur qui sera utilisé pour suivre toutes les
		// explorations à la suite
		Ocean sharks = new Ocean(fileName, "data/" + fileName, BasicDirections.values());

		System.out.println("Test plus court chemin evitant le plus de requins (v2) (avoid sharks)");
		sharks.reporters().add(new BasicOceanCanvas("Avoid Sharks - Shortest Path (v2)"))
				.exploreUsing(new AvoidSharksDequeue());
	}
}
