import java.io.FileNotFoundException;
import java.util.PriorityQueue;

import graphics.BasicOceanCanvas;
import ocean.BasicDirections;
import ocean.Coordinate;
import ocean.Direction;
import ocean.Ocean;

/***************************************
 * Question 4 : question complementaire
 ***************************************/

public class Bonus extends BackTrackingTraversal {

	@Override
	public boolean traverse(Ocean ocean, Coordinate start) {
		// TODO: implémenter le parcours trouvant le plus court chemin tout en evitant
		// le plus de requins (Question 4)

		// @OFF
		// On appele une des solutions proposées
		BreadthFirstAvoidSharks solution = new BreadthFirstAvoidSharks();
		// AvoidSharksDequeue solution = new AvoidSharksDequeue();

		return solution.traverse(ocean, start);
		/*@ON
		throw new Error("Méthode Bonus.traverse(Ocean, Coordinate) à implémenter (Question 4)");
		@OFF @ON */
	}

	/**
	 * Lance la visualisation du parcours de l'océan dont la carte se trouve dans
	 * {@code one-shark.txt}
	 * 
	 * @param args inutilisé
	 * @throws FileNotFoundException lorsque la carte n'est pas accessible
	 */
	public static void main(String[] args) throws FileNotFoundException {
		String fileName = (args.length < 1) ? "many-sharks.txt" : args[0];

		// on crée un nouvel océan à partir de la carte avec requins one-shark.txt
		// et on y associe un nouveau rapporteur qui sera utilisé pour suivre toutes les
		// explorations à la suite
		Ocean sharks = new Ocean(fileName, "data/" + fileName, BasicDirections.values());

		System.out.println("Test plus court chemin evitant le plus de requins (avoid sharks)");
		sharks.reporters().add(new BasicOceanCanvas("Avoid Sharks - Shortest Path")).exploreUsing(new Bonus());
	}
}

//@OFF

/*************
 * Solution 1
 *************/

/**
 * Une classe pour associer la distance au requin le plus proche et à Marin à
 * une cellule
 */
class CellShark implements Comparable<CellShark> {
	/**
	 * La cellule
	 */
	Coordinate cell;

	/**
	 * Distance à la case de départ (Marin)
	 */
	int startDist;

	/**
	 * Distance au requin le plus proche
	 */
	int sharkDist;

	/**
	 * Le constructeur
	 * 
	 * @param cell      la cellule
	 * @param startDist distance à la case de départ (Marin)
	 * @param sharkDist distance au requin le plus proche
	 */
	public CellShark(Coordinate cell, int startDist, int sharkDist) {
		this.cell = cell;
		this.startDist = startDist;
		this.sharkDist = sharkDist;
	}

	/**
	 * Comparaison avec un autre objet de la même classe.
	 * 
	 * <p>
	 * Compare les distinces aux requins, puis, en cas d'égalité, celles à Marin.
	 * </p>
	 */
	public int compareTo(CellShark c) {
		if (sharkDist != c.sharkDist)
			return sharkDist - c.sharkDist;
		else // sharkDist == c.sharkDist
			return startDist - c.startDist;
	}
}

/**
 * Parcours alternatif pour trouver le plus court chemin tout en évitant les
 * requins au maximum
 */
class BreadthFirstAvoidSharks extends BackTrackingTraversal {
	/**
	 * Implémente le parcours altérnatif pour trouver le plus court chemin tout en
	 * évitant les requins au maximum
	 */
	@Override
	public boolean traverse(Ocean ocean, Coordinate start) {
		PriorityQueue<CellShark> frontier = new PriorityQueue<CellShark>();
		frontier.add(new CellShark(start, 0, 0));

		ocean.setMark(start);

		CellShark current = null;
		while (!frontier.isEmpty()) {
			current = frontier.poll();
			if (ocean.isNemoAt(current.cell)) {
				backTrack(ocean, current.cell);
				return true;
			}

			for (Direction direction : ocean.directions()) {
				Coordinate neighbour = current.cell.moveTo(direction);

				if (ocean.isValid(neighbour) && !ocean.isWall(neighbour) && !ocean.isMarked(neighbour)) {
					ocean.setMark(neighbour, direction.getOpposite());
					frontier.add(new CellShark(neighbour, current.startDist + 1,
							(ocean.isThereASharkAt(neighbour) ? 1 : 0) + current.sharkDist));
				}
			}
		}

		return false;
	}
}

/*************
 * Solution 2
 *************/

class AvoidSharksDequeue extends BackTrackingTraversal {
	// Alternative Idea: use 3 queues: one for the current connected component of
	// maps \ sharks, one for upcoming sharks, and one for further sharks not yet
	// considered (still too far away)
	@Override
	public boolean traverse(Ocean ocean, Coordinate start) {
		LinkedList<Pair> now = new LinkedList<>();
		LinkedList<Pair> next = new LinkedList<>();
		LinkedList<Pair> sharks = new LinkedList<>();

		now.add(new Pair(start, 0));
		Pair current = null;

		ocean.setMark(start);

		do {
			while (!now.isEmpty() || !sharks.isEmpty()) {
				// look for next cell (transfer shark if necessary)
				if (now.isEmpty())
					now.add(sharks.poll());

				current = now.poll();

				// while distance to next shark is reached, enqueue it
				while (!sharks.isEmpty() && current.counter >= sharks.peek().counter)
					now.add(sharks.poll());

				// test success
				if (ocean.isNemoAt(current.cell)) {
					backTrack(ocean, current.cell);
					return true;
				}

				// enqueue relevant neighboring cells
				for (Direction direction : ocean.directions()) {
					Coordinate neighbour = current.cell.moveTo(direction);

					if (ocean.isValid(neighbour) && !ocean.isWall(neighbour) && !ocean.isMarked(neighbour)) {
						ocean.setMark(neighbour, direction.getOpposite());
						if (ocean.isThereASharkAt(neighbour))
							next.add(new Pair(neighbour, current.counter + 1));
						else
							now.add(new Pair(neighbour, current.counter + 1));
					}
				}
			}
			// upon termination, update sharks and cells queues
			sharks = next;
			next = new LinkedList<Pair>();
			now = new LinkedList<Pair>();
		} while (!sharks.isEmpty());
		return false;
	}
}