import java.io.FileNotFoundException;
import java.util.PriorityQueue;

import graphics.BasicOceanCanvas;
import ocean.BasicDirections;
import ocean.Coordinate;
import ocean.Direction;
import ocean.Ocean;

/**
 * Parcours altérnatif pour trouver le plus court chemin tout en évitant les
 * requins au maximum
 */
class BreadthFirstAvoidSharks extends BackTrackingTraversal {
	/**
	 * Implémente le parcours altérnatif pour trouver le plus court chemin tout en
	 * évitant les requins au maximum
	 */
	@Override
	public boolean traverse(Ocean ocean, Coordinate start) {
		// TODO: implémenter un parcours qui calcule un chemin de longueur optimale
		// 		 parmi ceux qui rencontrent le moins de requins
		// (Question 4)
		// @OFF
		PriorityQueue<CellShark> frontier = new PriorityQueue<CellShark>();
		frontier.add(new CellShark(start, 0, 0));
		
		ocean.setMark(start);
		
		CellShark current = null;
		while (!frontier.isEmpty()) {
			current = frontier.poll();
			if (ocean.isNemoAt(current.cell)) {
				backTrack(ocean, current.cell);
				return true;
			}

			for (Direction direction : ocean.directions()) {
				Coordinate neighbour = current.cell.moveTo(direction);

				if (ocean.isValid(neighbour) && !ocean.isWall(neighbour) && !ocean.isMarked(neighbour)) {
					ocean.setMark(neighbour, direction.getOpposite());
					frontier.add(new CellShark(neighbour, current.startDist + 1,
							(ocean.isThereASharkAt(neighbour) ? 1 : 0) + current.sharkDist));
				}
			}
		}

		return false;
		/* @ON
		throw new Error("Méthode BreadthFirstAvoidSharks.traverse(Ocean, Coordinate) à implémenter (Question 4)");
		@OFF @ON */
	}
	
	/**
	 * Lance la visualisation du parcours de l'océan dont la carte se trouve dans
	 * {@code one-shark.txt}
	 * 
	 * @param args non utilisé
	 * @throws FileNotFoundException lorsque la carte n'est pas accessible
	 */
	public static void main(String[] args) throws FileNotFoundException {
		String fileName = (args.length < 1) ? "one-shark.txt" : args[0];

		// on crée un nouveau océan à partir de la carte avec requins one-shark.txt
		// et on y associe un nouveau rapporteur qui sera utilisé pour suivre toutes les
		// explorations à la suite
		Ocean sharks = new Ocean(fileName, "data/" + fileName, BasicDirections.values());

		System.out.println("Test plus court chemin evitant le plus de requins (avoid sharks)");
		sharks.reporters().add(new BasicOceanCanvas("Avoid Sharks - Shortest Path"))
				.exploreUsing(new BreadthFirstAvoidSharks());
	}
}
