import java.io.FileNotFoundException;

import graphics.BasicOceanCanvas;
import ocean.BasicDirections;
import ocean.Coordinate;
import ocean.Direction;
import ocean.Ocean;
import ocean.Traversal;

/**************************************
 * Question 1 : parcours en profondeur
 **************************************/

/**
 * Parcours en profondeur
 */
class DepthFirst extends Traversal {
	/**
	 * Implémente le parcours en profondeur
	 */
	@Override
	public boolean traverse(Ocean ocean, Coordinate start) {
		// TODO: implémenter le parcours en profondeur (Question 1)

		// @OFF
		// Nous avons trouvé Nemo
		if (ocean.isNemoAt(start))
			return true;
		
		// Ici, on est sûr que start est une case vide, non encore explorée
		// On la marque désormais comme ayant été explorée
		ocean.setMark(start);

		// Puis on explore tous les voisins récursivement
		for (Direction direction : ocean.directions()) {
			Coordinate neighbour = start.moveTo(direction);

			// On évite les cellules hors océan
			// les mur et les cases déjà explorées				
			// L'appel récursif à traverse() renvoit true lorsque Nemo a été trouvé
			if (ocean.isValid(neighbour) 
				&& !ocean.isWall(neighbour)
				&& !ocean.isMarked(neighbour)
				&& traverse(ocean, neighbour)) {
				ocean.setMark(start, path);
				return true;
			}
			// Si l'appel récursif à traverse() a renvoyé false --
			// on continue à chercher passant par d'autres cases voisines
		}

		// Ici, on a exploré tous les chemins partant de start sans avoir trouvé Nemo
		// On est, donc, dans un cul-de-sac -- il faut revenir sur ses pas
		ocean.setMark(start, deadEnd);
		return false;
		/* @ON
		throw new Error("Méthode DepthFirst.traverse(Ocean, Coordinate) à implémenter (Question 1)");
		@OFF @ON */
	}

	/**
	 * Lance la visualisation du parcours de l'océan dont la carte se trouve dans
	 * {@code no-sharks.txt}
	 * 
	 * @param args inutilisé
	 * @throws FileNotFoundException lorsque la carte n'est pas accessible
	 */
	public static void main(String[] args) throws FileNotFoundException {
		String fileName = (args.length < 1) ? "no-sharks.txt" : args[0];

		// on crée un nouvel océan à partir de la carte sans requins
		// et on y associe un nouveau rapporteur qui sera utilisé pour suivre toutes les
		// explorations à la suite
		Ocean noSharks = new Ocean(fileName, "data/" + fileName, BasicDirections.values());

		// on lance le parcours en profondeur
		System.out.println("Test parcours en profondeur (depth first search)");
		noSharks.reporters().add(new BasicOceanCanvas("No Sharks - DFS")).exploreUsing(new DepthFirst());
	}
}
